#include <avr/io.h>
#include <util/delay.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "gsm/gsm.h"
#include "nokia5110/nokia5110.h"
#include "mpu6050/mpu6050.h"

#define LCD_WIDTH 	84
#define LCD_HEIGHT	48

#define OX			0
#define OY			1
#define OZ			2
#define marja			0.5

double limitOX = 0;
double limitOY = 0;
double limitOZ = 0;


double get_value(int dir) {
	double axg = 0, ayg = 0, azg = 0;
	double gxds = 0, gyds = 0, gzds = 0;

	mpu6050_getConvData(&axg, &ayg, &azg, &gxds, &gyds, &gzds);

	if (dir == OX) {
		return axg;
	} else {
		if (dir == OY) {
			return ayg;
		} else {
			return azg;
		}
	} 
}


void get_initial_values(){
	int i;
	for(i = 0; i< 50; i++) {
		limitOX += get_value(OX);
		limitOZ += get_value(OZ);
		limitOY += get_value(OY);
	}

	limitOX /= 50;
	limitOY /= 50;
	limitOZ /= 50;
	
}

int check_falling(double ox, double oy, double oz) {

	if (ox < -marja || ox > marja)
		return 1;

	if (oy < -marja || oy > marja)
		return 1;


	return 0;

}

void print_axes(double ox, double oy, double oz) {
	char buff[10];

	nokia_lcd_clear();

	nokia_lcd_set_cursor(0,0);
	snprintf(buff, 10, "OX: %d", ox);
	nokia_lcd_write_string(buff, 1);

	nokia_lcd_set_cursor(0, 12);
	snprintf(buff, 10, "OY: %d", oy);
	nokia_lcd_write_string(buff, 1);

	nokia_lcd_set_cursor(0, 24);
	snprintf(buff, 10, "OZ: %d", oz);
	nokia_lcd_write_string(buff, 1);


	nokia_lcd_render();
	_delay_ms(3000);
}

void print_start_screen(void) {
	nokia_lcd_clear();

	nokia_lcd_set_cursor(0, 0);
	nokia_lcd_write_string("Hristu Cristina", 1);

	nokia_lcd_set_cursor(20, 12);
	nokia_lcd_write_string("Andrada", 1);

	nokia_lcd_set_cursor(20, 24);
	nokia_lcd_write_string("335 CA", 1);

	nokia_lcd_set_cursor(3, 36);
	nokia_lcd_write_string("Emergency dev", 1);

	nokia_lcd_render();

	_delay_ms(10000);
}



int main(void){

	double ox = 0, oy = 0, oz = 0;
	
	nokia_lcd_init();
	mpu6050_init();

 	print_start_screen();
	get_initial_values();

	while(1) {

		ox = get_value(OX);
		oy = get_value(OY);
		oz = get_value(OZ);

		print_axes(ox, oy, oz);
		
		if(check_falling(ox, oy, oz) == 1) {

			nokia_lcd_clear();
			nokia_lcd_set_cursor(10, 24);
			nokia_lcd_write_string("SMS!!!", 2);
			nokia_lcd_render();
			_delay_ms(10000);

			break;
			
		}

	
	}

	return 0;

}
